@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapagg.h 8984 2009-05-01 07:06:05Z tbonfort $
+ * $Id: mapagg.h 10743 2010-11-19 17:05:38Z tbonfort $
  *
  * Project:  MapServer
  * Purpose:  AGG template library types.
@@ -27,126 +27,8 @@
  * DEALINGS IN THE SOFTWARE.
  ******************************************************************************/
 
-#include "renderers/agg/include/agg_array.h"
 #include "renderers/agg/include/agg_rendering_buffer.h"
 
-
-template<class T> class mapserv_row_ptr_cache
-{
-public:
-	typedef mapserver::const_row_info<T> row_data;
-
-	mapserv_row_ptr_cache() :
-            m_buf(0),
-            m_rows(),
-            m_width(0),
-            m_height(0),
-            m_stride(0)
-	{
-	}
-	
-	mapserv_row_ptr_cache( const gdImagePtr pImg ) :
-            m_buf(0),
-            m_rows(),
-            m_width(0),
-            m_height(0),
-            m_stride(0)
-	{
-		attach( pImg->tpixels, pImg->sx, pImg->sy, pImg->sx * sizeof( T ) );
-	}
-	
-	void attach( T **ppRows, unsigned width, unsigned height, int stride )
-	{
-		m_width	= width;
-		m_height = height;
-		m_stride = stride;
-		
-		if( height > m_rows.size() )
-			m_rows.resize( height );
-			
-		T** rows	= &m_rows[0];
-		int iRowIndex = 0;
-
-		while( height--)
-		{
-			*rows = ppRows[iRowIndex];
-			rows++;
-			iRowIndex++;
-		}
-	}
-
-        /*-------------------------------------------------------------------*/
-        AGG_INLINE       T* buf()          { return m_buf;    }
-        AGG_INLINE const T* buf()    const { return m_buf;    }
-        AGG_INLINE unsigned width()  const { return m_width;  }
-        AGG_INLINE unsigned height() const { return m_height; }
-        AGG_INLINE int      stride() const { return m_stride; }
-        AGG_INLINE unsigned stride_abs() const 
-        {
-            return (m_stride < 0) ? unsigned(-m_stride) : unsigned(m_stride); 
-        }
-
-        /*-------------------------------------------------------------------*/
-        AGG_INLINE       T* row_ptr(int, int y, unsigned) 
-        { 
-            return m_rows[y]; 
-        }
-        AGG_INLINE       T* row_ptr(int y)       { return m_rows[y]; }
-        AGG_INLINE const T* row_ptr(int y) const { return m_rows[y]; }
-        AGG_INLINE row_data row    (int y) const 
-        { 
-            return row_data(0, m_width-1, m_rows[y]); 
-        }
-
-        /*-------------------------------------------------------------------*/
-        T const* const* rows() const { return &m_rows[0]; }
-
-        /*-------------------------------------------------------------------*/
-        template<class RenBuf>
-        void copy_from(const RenBuf& src)
-        {
-            unsigned h = height();
-            if(src.height() < h) h = src.height();
-        
-            unsigned l = stride_abs();
-            if(src.stride_abs() < l) l = src.stride_abs();
-        
-            l *= sizeof(T);
-
-            unsigned y;
-            unsigned w = width();
-            for (y = 0; y < h; y++)
-            {
-                memcpy(row_ptr(0, y, w), src.row_ptr(y), l);
-            }
-        }
-
-        /*-------------------------------------------------------------------*/
-        void clear(T value)
-        {
-            unsigned y;
-            unsigned w = width();
-            unsigned stride = stride_abs();
-            for(y = 0; y < height(); y++)
-            {
-                T* p = row_ptr(0, y, w);
-                unsigned x;
-                for(x = 0; x < stride; x++)
-                {
-                    *p++ = value;
-                }
-            }
-        }
-
-    private:
-        /*-------------------------------------------------------------------*/
-        T*            m_buf;        /* Pointer to rendering buffer */
-        mapserver::pod_array<T*> m_rows;  /* Pointers to each row of the buffer */
-        unsigned      m_width;      /* Width in pixels */
-        unsigned      m_height;     /* Height in pixels */
-        int           m_stride;     /* Number of bytes per row. Can be < 0 */
-};
-
 /*
  * interface to a shapeObj representing lines, providing the functions
  * needed by the agg rasterizer. treats shapeObjs with multiple linestrings.
