@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maperror.c 11755 2011-05-25 05:04:09Z sdlime $
+ * $Id: maperror.c 11415 2011-03-31 08:50:39Z tbonfort $
  *
  * Project:  MapServer
  * Purpose:  Implementation of msSetError(), msDebug() and related functions.
@@ -41,7 +41,7 @@
 #endif
 #include <stdarg.h>
 
-MS_CVSID("$Id: maperror.c 11755 2011-05-25 05:04:09Z sdlime $")
+MS_CVSID("$Id: maperror.c 11415 2011-03-31 08:50:39Z tbonfort $")
 
 static char *ms_errorCodes[MS_NUMERRORCODES] = {"",
 						"Unable to access file.",
@@ -80,17 +80,19 @@ static char *ms_errorCodes[MS_NUMERRORCODES] = {"",
 						"Invalid rectangle.",
 						"Date/time error.",
 						"GML encoding error.",
-            "SOS server error.",
+						"SOS server error.",
 						"NULL parent pointer error.",
-            "AGG library error.",
-            "OWS error."
+						"AGG library error.",
+						"OWS error.",
+						"OpenGL renderer error.",
+						"Renderer error."
 };
 
 #ifndef USE_THREAD
 
 errorObj *msGetErrorObj()
 {
-    static errorObj ms_error = {MS_NOERR, "", "", NULL};
+    static errorObj ms_error = {MS_NOERR, "", "", MS_FALSE, NULL};
 
     return &ms_error;
 }
@@ -199,11 +201,13 @@ static errorObj *msInsertErrorObj(void)
       {
           new_error->next = ms_error->next;
           new_error->code = ms_error->code;
-          strcpy(new_error->routine, ms_error->routine);
-          strcpy(new_error->message, ms_error->message);
+          new_error->isreported = ms_error->isreported;
+          strlcpy(new_error->routine, ms_error->routine, sizeof(new_error->routine));
+          strlcpy(new_error->message, ms_error->message, sizeof(new_error->message));
 
           ms_error->next = new_error;
           ms_error->code = MS_NOERR;
+          ms_error->isreported = MS_FALSE;
           ms_error->routine[0] = '\0';
           ms_error->message[0] = '\0';
       }
@@ -326,8 +330,7 @@ void msSetError(int code, const char *message_fmt, const char *routine, ...)
   if(!routine)
     strcpy(ms_error->routine, "");
   else {
-    strncpy(ms_error->routine, routine, ROUTINELENGTH);
-    ms_error->routine[ROUTINELENGTH-1] = '\0';
+    strlcpy(ms_error->routine, routine, sizeof(ms_error->routine));
   }
 
   if(!message_fmt)
@@ -351,6 +354,7 @@ void msWriteError(FILE *stream)
   while (ms_error && ms_error->code != MS_NOERR)
   {
       msIO_fprintf(stream, "%s: %s %s <br>\n", ms_error->routine, ms_errorCodes[ms_error->code], ms_error->message);
+      ms_error->isreported = MS_TRUE;
       ms_error = ms_error->next;
   }
 }
@@ -366,6 +370,7 @@ void msWriteErrorXML(FILE *stream)
 
       msIO_fprintf(stream, "%s: %s %s\n", ms_error->routine, 
                    ms_errorCodes[ms_error->code], message);
+      ms_error->isreported = MS_TRUE;
       ms_error = ms_error->next;
 
       msFree(message);
@@ -373,9 +378,10 @@ void msWriteErrorXML(FILE *stream)
 }
 
 void msWriteErrorImage(mapObj *map, char *filename, int blank) {
-  gdFontPtr font = gdFontSmall;
-  imageObj img;
-  int width=400, height=300, color;
+  imageObj *img;
+  rendererVTableObj *renderer;
+  int font_index = 0;
+  int width=400, height=300;
   int nMargin =5;
   int nTextLength = 0;
   int nUsableWidth = 0;
@@ -389,12 +395,15 @@ void msWriteErrorImage(mapObj *map, char *filename, int blank) {
   int nXPos = 0;
   int nYPos = 0;
   int nWidthTxt = 0;
-  int nSpaceBewteenLines = font->h;
-  int nBlack = 0;   
   outputFormatObj *format = NULL;
   char *errormsg = msGetErrorString("; ");
-  char *pFormatBuffer;
-  char cGDFormat[128];
+  fontMetrics *font = NULL;
+  char *imagepath = NULL, *imageurl = NULL;
+  labelStyleObj ls;
+  colorObj labelcolor, labeloutlinecolor, imagecolor, *imagecolorptr=NULL;
+  ls.color = &labelcolor;
+  ls.outlinecolor = &labeloutlinecolor;
+  
   if (map) {
       if( map->width > 0 && map->height > 0 )
       {
@@ -402,97 +411,91 @@ void msWriteErrorImage(mapObj *map, char *filename, int blank) {
           height = map->height;
       }
       format = map->outputformat;
+      imagepath = map->web.imagepath;
+      imageurl = map->web.imageurl;
   }
 
   /* Default to GIF if no suitable GD output format set */
-  if (format == NULL || (!MS_DRIVER_GD(format) && !MS_DRIVER_AGG(format))) 
-    format = msCreateDefaultOutputFormat( NULL, "GD/PC256" );
+  if (format == NULL || !MS_RENDERER_PLUGIN(format) || !format->vtable->supports_bitmap_fonts) 
+    format = msCreateDefaultOutputFormat( NULL, "GD/PC256", "gif" );
 
-  if(format->imagemode == MS_IMAGEMODE_RGB || format->imagemode == MS_IMAGEMODE_RGBA) {
-    img.img.gd = gdImageCreateTrueColor(width, height);
-        gdImageAlphaBlending(img.img.gd, 0);
-    color = gdTrueColor(map->imagecolor.red,
-                        map->imagecolor.green,
-                        map->imagecolor.blue); /* BG color */
-    nBlack = gdTrueColor(0,0,0); /* Text color */
-
-    gdImageFilledRectangle(img.img.gd, 0, 0, width, height, color);
-
-  } else {
-    img.img.gd = gdImageCreate(width, height);
-    color = gdImageColorAllocate(img.img.gd, map->imagecolor.red, 
-                                 map->imagecolor.green,
-                                 map->imagecolor.blue); /* BG color */
-    nBlack = gdImageColorAllocate(img.img.gd, 0,0,0); /* Text color */
+  if(!format->transparent) {
+     if(map && MS_VALID_COLOR(map->imagecolor)) {
+        imagecolorptr = &map->imagecolor;
+     } else {
+         MS_INIT_COLOR(imagecolor,255,255,255,255);
+         imagecolorptr = &imagecolor;
+     }
   }
 
-  if (map->outputformat && map->outputformat->transparent)
-    gdImageColorTransparent(img.img.gd, color);
+  img = msImageCreate(width,height,format,imagepath,imageurl,MS_DEFAULT_RESOLUTION,MS_DEFAULT_RESOLUTION,imagecolorptr);
+  renderer = MS_IMAGE_RENDERER(img);
 
-
-  nTextLength = strlen(errormsg); 
-  nWidthTxt  =  nTextLength * font->w;
-  nUsableWidth = width - (nMargin*2);
-
-  /* Check to see if it all fits on one line. If not, split the text on several lines. */
-  if(!blank) {
-    if (nWidthTxt > nUsableWidth) {
-      nMaxCharsPerLine =  nUsableWidth/font->w;
-      nLines = (int) ceil ((double)nTextLength / (double)nMaxCharsPerLine);
-      if (nLines > 0) {
-        papszLines = (char **)malloc(nLines*sizeof(char *));
-        for (i=0; i<nLines; i++) {
-          papszLines[i] = (char *)malloc((nMaxCharsPerLine+1)*sizeof(char));
-          papszLines[i][0] = '\0';
-        }
-      }
-      for (i=0; i<nLines; i++) {
-        nStart = i*nMaxCharsPerLine;
-        nEnd = nStart + nMaxCharsPerLine;
-        if (nStart < nTextLength) {
-          if (nEnd > nTextLength)
-            nEnd = nTextLength;
-          nLength = nEnd-nStart;
-
-          strncpy(papszLines[i], errormsg+nStart, nLength);
-          papszLines[i][nLength] = '\0';
-        }
-      }
-    } else {
-      nLines = 1;
-      papszLines = (char **)malloc(nLines*sizeof(char *));
-      papszLines[0] = strdup(errormsg);
-    }   
-    for (i=0; i<nLines; i++) {
-      nYPos = (nSpaceBewteenLines) * ((i*2) +1); 
-      nXPos = nSpaceBewteenLines;
-
-      gdImageString(img.img.gd, font, nXPos, nYPos, (unsigned char *)papszLines[i], nBlack);
-    }
-    if (papszLines) {
-      for (i=0; i<nLines; i++) {
-	free(papszLines[i]);
-      }
-      free(papszLines);
-    }
+  for(i=0;i<5;i++) {
+	  /* use the first font we find */
+	  if((font = renderer->bitmapFontMetrics[font_index]) != NULL) {
+	     ls.size = i;
+         MS_INIT_COLOR(*ls.color,0,0,0,255);
+         MS_INIT_COLOR(*ls.outlinecolor,255,255,255,255);
+         break;
+	  }
+  }
+  /* if no font found we can't do much. this shouldn't happen */
+  if(font) {
+	  
+	  nTextLength = strlen(errormsg); 
+	  nWidthTxt  =  nTextLength * font->charWidth;
+	  nUsableWidth = width - (nMargin*2);
+	
+	  /* Check to see if it all fits on one line. If not, split the text on several lines. */
+	  if(!blank) {
+		if (nWidthTxt > nUsableWidth) {
+		  nMaxCharsPerLine =  nUsableWidth/font->charWidth;
+		  nLines = (int) ceil ((double)nTextLength / (double)nMaxCharsPerLine);
+		  if (nLines > 0) {
+			papszLines = (char **)malloc(nLines*sizeof(char *));
+			for (i=0; i<nLines; i++) {
+			  papszLines[i] = (char *)malloc((nMaxCharsPerLine+1)*sizeof(char));
+			  papszLines[i][0] = '\0';
+			}
+		  }
+		  for (i=0; i<nLines; i++) {
+			nStart = i*nMaxCharsPerLine;
+			nEnd = nStart + nMaxCharsPerLine;
+			if (nStart < nTextLength) {
+			  if (nEnd > nTextLength)
+				nEnd = nTextLength;
+			  nLength = nEnd-nStart;
+	
+			  strncpy(papszLines[i], errormsg+nStart, nLength);
+			  papszLines[i][nLength] = '\0';
+			}
+		  }
+		} else {
+		  nLines = 1;
+		  papszLines = (char **)malloc(nLines*sizeof(char *));
+		  papszLines[0] = msStrdup(errormsg);
+		}   
+		for (i=0; i<nLines; i++) {
+		  nYPos = (font->charHeight) * ((i*2) +1); 
+		  nXPos = font->charWidth;;
+		  renderer->renderBitmapGlyphs(img, nXPos, nYPos, &ls, papszLines[i]); 
+		}
+		if (papszLines) {
+		  for (i=0; i<nLines; i++) {
+		free(papszLines[i]);
+		  }
+		  free(papszLines);
+		}
+	  }
   }
 
   /* actually write the image */
-  if(!filename) 
+  if(!filename) {
       msIO_printf("Content-type: %s%c%c", MS_IMAGE_MIME_TYPE(format), 10,10);
-  if (MS_DRIVER_GD(format))
-    msSaveImageGD(&img, filename, format);
-  else
-  {
-      pFormatBuffer = format->driver;
-      strcpy(cGDFormat, "gd/");
-      strcat(cGDFormat, &(format->driver[4]));
-      format->driver = &cGDFormat[0];
-      msSaveImageGD(&img, filename, format);
-      format->driver = pFormatBuffer;
   }
-
-  gdImageDestroy(img.img.gd);
+  msSaveImage(NULL,img,filename);
+  msFreeImage(img);
 
   if (format->refcount == 0)
     msFreeOutputFormat(format);
@@ -513,22 +516,16 @@ char *msGetVersion() {
 #ifdef USE_GD_JPEG
   strcat(version, " OUTPUT=JPEG");
 #endif
-#ifdef USE_GD_WBMP
-  strcat(version, " OUTPUT=WBMP");
-#endif
 #ifdef USE_PDF
   strcat(version, " OUTPUT=PDF");
 #endif
-#ifdef USE_MING_FLASH
-  strcat(version, " OUTPUT=SWF");
+#ifdef USE_KML
+  strcat(version, " OUTPUT=KML");
 #endif
-  strcat(version, " OUTPUT=SVG");
 #ifdef USE_PROJ
   strcat(version, " SUPPORTS=PROJ");
 #endif
-#ifdef USE_AGG
   strcat(version, " SUPPORTS=AGG");
-#endif
 #ifdef USE_CAIRO
   strcat(version, " SUPPORTS=CAIRO");
 #endif
@@ -574,9 +571,6 @@ char *msGetVersion() {
 #ifdef USE_POINT_Z_M
   strcat(version, " SUPPORTS=POINT_Z_M");
 #endif
-#ifdef USE_RGBA_PNG
-  strcat(version, " SUPPORTS=RGBA_PNG");
-#endif
 #ifdef USE_TIFF
   strcat(version, " INPUT=TIFF");
 #endif
@@ -600,9 +594,6 @@ char *msGetVersion() {
 #endif
 #ifdef USE_GDAL
   strcat(version, " INPUT=GDAL");
-#endif
-#ifdef USE_MYGIS
-  strcat(version, " INPUT=MYGIS");
 #endif
   strcat(version, " INPUT=SHAPEFILE");
   return(version);
