@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapgraticule.c 10105 2010-04-15 12:17:13Z assefa $
+ * $Id: mapgraticule.c 11881 2011-07-07 19:55:43Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  Graticule Renderer
@@ -30,7 +30,7 @@
 #include <assert.h>
 #include "mapproject.h"
 
-MS_CVSID("$Id: mapgraticule.c 10105 2010-04-15 12:17:13Z assefa $")
+MS_CVSID("$Id: mapgraticule.c 11881 2011-07-07 19:55:43Z sdlime $")
 
 /**********************************************************************************************************************
  *
@@ -89,19 +89,19 @@ int msGraticuleLayerOpen(layerObj *layer)
     pInfo->blabelaxes = 1;
   
   if( pInfo->labelformat == NULL ) {
-    pInfo->labelformat = (char *) malloc( strlen( MAPGRATICULE_FORMAT_STRING_DEFAULT ) + 1 );
+    pInfo->labelformat = (char *) msSmallMalloc( strlen( MAPGRATICULE_FORMAT_STRING_DEFAULT ) + 1 );
     pInfo->ilabeltype = (int) lpDefault;
     strcpy( pInfo->labelformat, MAPGRATICULE_FORMAT_STRING_DEFAULT );
   } else if( strcmp( pInfo->labelformat, "DDMMSS" ) == 0 ) {
-    pInfo->labelformat = (char *) malloc( strlen( MAPGRATICULE_FORMAT_STRING_DDMMSS ) + 1 );
+    pInfo->labelformat = (char *) msSmallMalloc( strlen( MAPGRATICULE_FORMAT_STRING_DDMMSS ) + 1 );
     pInfo->ilabeltype = (int) lpDDMMSS;
     strcpy( pInfo->labelformat, MAPGRATICULE_FORMAT_STRING_DDMMSS );
   } else if( strcmp( pInfo->labelformat, "DDMM" )   == 0 ) {
-    pInfo->labelformat = (char *) malloc( strlen( MAPGRATICULE_FORMAT_STRING_DDMM ) + 1 );
+    pInfo->labelformat = (char *) msSmallMalloc( strlen( MAPGRATICULE_FORMAT_STRING_DDMM ) + 1 );
     pInfo->ilabeltype = (int) lpDDMM;
     strcpy( pInfo->labelformat, MAPGRATICULE_FORMAT_STRING_DDMM );
   } else if( strcmp( pInfo->labelformat, "DD" )   == 0 ) {
-    pInfo->labelformat = (char *) malloc( strlen( MAPGRATICULE_FORMAT_STRING_DD ) + 1 );
+    pInfo->labelformat = (char *) msSmallMalloc( strlen( MAPGRATICULE_FORMAT_STRING_DD ) + 1 );
     pInfo->ilabeltype = (int) lpDD;
     strcpy( pInfo->labelformat, MAPGRATICULE_FORMAT_STRING_DD );
   }
@@ -142,14 +142,17 @@ int msGraticuleLayerClose(layerObj *layer)
     free( pInfo->pboundinglines );
     pInfo->pboundinglines = NULL;
   }
-  
+
+  free(layer->layerinfo);
+  layer->layerinfo=NULL;
+
   return MS_SUCCESS;
 }
 
 /**********************************************************************************************************************
  *
  */
-int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect)
+int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
 {
   graticuleObj *pInfo = (graticuleObj *) layer->layerinfo;
   int iAxisTickCount = 0;
@@ -203,10 +206,11 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect)
    * These lines will be used when generating labels to get correct placement at arc/rect edge intersections.
    */
   rectMapCoordinates = layer->map->extent;
-  pInfo->pboundinglines   = (lineObj *)  malloc( sizeof( lineObj )  * 4 );
-  pInfo->pboundingpoints = (pointObj *) malloc( sizeof( pointObj ) * 8 );
-
+  pInfo->pboundinglines   = (lineObj *)  msSmallMalloc( sizeof( lineObj )  * 4 );
+  pInfo->pboundingpoints = (pointObj *) msSmallMalloc( sizeof( pointObj ) * 8 );
+  
   {
+
     /*
      * top
      */
@@ -291,8 +295,8 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
 
   shape->numlines = 1;
   shape->type = MS_SHAPE_LINE;
-  shape->line = (lineObj *) malloc(sizeof( lineObj ));
-  shape->line->numpoints = (int) pInfo->maxsubdivides;   
+  shape->line = (lineObj *) msSmallMalloc(sizeof( lineObj ));
+  shape->line->numpoints = (int) pInfo->maxsubdivides; 
 
   /*
    * Subdivide and draw current arc, rendering the arc labels first
@@ -317,7 +321,7 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
 
       dStartY = (pInfo->pboundinglines[1].point[1].y - pInfo->pboundinglines[1].point[0].y) * dDeltaX + pInfo->pboundinglines[1].point[0].y;
       shape->line->numpoints = (int) 2;
-      shape->line->point = (pointObj *) malloc( sizeof( pointObj ) * 2 );
+      shape->line->point = (pointObj *) msSmallMalloc( sizeof( pointObj ) * 2 );
             
       shape->line->point[0].x = pInfo->dwhichlongitude;
       shape->line->point[0].y = dStartY;
@@ -344,7 +348,7 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
 
       dStartY = (pInfo->pboundinglines[0].point[1].y - pInfo->pboundinglines[0].point[0].y) * dDeltaX + pInfo->pboundinglines[0].point[1].y;
       shape->line->numpoints = (int) 2;
-      shape->line->point = (pointObj *) malloc( sizeof( pointObj ) * 2 );
+      shape->line->point = (pointObj *) msSmallMalloc( sizeof( pointObj ) * 2 );
 
       shape->line->point[0].x = pInfo->dwhichlongitude;
       shape->line->point[0].y = dStartY - dArcDelta;
@@ -360,7 +364,7 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
 
     case 2:
       shape->line->numpoints = (int) shape->line->numpoints + 1;
-      shape->line->point = (pointObj *) malloc( sizeof( pointObj ) * shape->line->numpoints );
+      shape->line->point = (pointObj *) msSmallMalloc( sizeof( pointObj ) * shape->line->numpoints );
 
       shape->line->point[0].x = pInfo->dwhichlongitude;
       shape->line->point[0].y = pInfo->dstartlatitude;
@@ -373,6 +377,7 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
       }
             
       pInfo->ilabelstate = 0;
+
       pInfo->dwhichlongitude      += pInfo->dincrementlongitude;
       break;
 
@@ -381,7 +386,7 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
       break;
     }
       
-  } else {
+  } else { /*horizontal*/
     int iPointIndex;
     double dArcDelta = (pInfo->dendlongitude - pInfo->dstartlongitude) / (double) shape->line->numpoints;
     double dArcPosition   = pInfo->dstartlongitude + dArcDelta;
@@ -401,7 +406,7 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
 
       dStartX = (pInfo->pboundinglines[2].point[1].x - pInfo->pboundinglines[2].point[0].x) * dDeltaY + pInfo->pboundinglines[2].point[0].x;
       shape->line->numpoints = (int) 2;
-      shape->line->point    = (pointObj *) malloc( sizeof( pointObj ) * 2 );
+      shape->line->point    = (pointObj *) msSmallMalloc( sizeof( pointObj ) * 2 );
             
       shape->line->point[0].x = dStartX;
       shape->line->point[0].y = pInfo->dwhichlatitude;
@@ -428,7 +433,7 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
 
       dStartX = (pInfo->pboundinglines[3].point[1].x - pInfo->pboundinglines[3].point[0].x) * dDeltaY + pInfo->pboundinglines[3].point[0].x;
       shape->line->numpoints = (int) 2;
-      shape->line->point = (pointObj *) malloc( sizeof( pointObj ) * 2 );
+      shape->line->point = (pointObj *) msSmallMalloc( sizeof( pointObj ) * 2 );
 
       shape->line->point[0].x = dStartX - dArcDelta;
       shape->line->point[0].y = pInfo->dwhichlatitude;
@@ -444,7 +449,7 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
 
     case 2:
       shape->line->numpoints = (int) shape->line->numpoints + 1;
-      shape->line->point = (pointObj *) malloc( sizeof( pointObj ) * shape->line->numpoints );
+      shape->line->point = (pointObj *) msSmallMalloc( sizeof( pointObj ) * shape->line->numpoints );
 
       shape->line->point[0].x = pInfo->dstartlongitude;
       shape->line->point[0].y = pInfo->dwhichlatitude;
@@ -466,7 +471,7 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
     }
   }
 
-  /*
+ /*
    * Increment and move to next arc
    */
  
@@ -475,10 +480,11 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
       pInfo->bvertical   = 0;
   }
 
-  if (pInfo->dwhichlatitude >  pInfo->dendlatitude)
+  if (pInfo->dwhichlatitude >  pInfo->dendlatitude) {
+    /* free the lineObj and pointObj that have been erroneously allocated beforehand */
+    msFreeShape(shape);
     return MS_DONE;
-
-   
+  }
 
   return MS_SUCCESS;
 }
@@ -488,9 +494,9 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
  */
 int msGraticuleLayerGetItems(layerObj *layer)
 {
-  char **ppItemName   = (char **) malloc( sizeof( char * ) );
+  char **ppItemName   = (char **) msSmallMalloc( sizeof( char * ) );
 
-  *ppItemName = (char *) malloc( 64 ); /* why is this necessary? */
+  *ppItemName = (char *) msSmallMalloc( 64 ); /* why is this necessary? */
   strcpy( *ppItemName, "Graticule" );
 
   layer->numitems   = 1;
@@ -518,9 +524,9 @@ void msGraticuleLayerFreeItemInfo(layerObj *layer)
 /**********************************************************************************************************************
  *
  */
-int msGraticuleLayerGetShape(layerObj *layer, shapeObj *shape, int tile, long record)
+int msGraticuleLayerGetShape(layerObj *layer, shapeObj *shape, resultObj *record)
 {
-  return 0;
+  return MS_FAILURE;
 }
 
 /**********************************************************************************************************************
@@ -541,7 +547,7 @@ int msGraticuleLayerGetExtent(layerObj *layer, rectObj *extent)
 /**********************************************************************************************************************
  *
  */
-int msGraticuleLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c, int tile, long record)
+int msGraticuleLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c, shapeObj* shape)
 {
   return MS_SUCCESS;
 }
@@ -639,7 +645,8 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
     if (map->cellsize == 0)
       map->cellsize = msAdjustExtent(&(map->extent),map->width,map->height);
 
-    psValues = (graticuleIntersectionObj *)malloc(sizeof(graticuleIntersectionObj));
+    psValues = (graticuleIntersectionObj *)msSmallMalloc(sizeof(graticuleIntersectionObj));
+ 
     msGraticuleLayerInitIntersectionPoints(psValues);
 
     if(layer->transform == MS_TRUE)
@@ -657,7 +664,7 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
  
     msLayerOpen(layer);
    
-    status = msLayerWhichShapes(layer, searchrect);
+    status = msLayerWhichShapes(layer, searchrect, MS_FALSE);
     if(status == MS_DONE) { /* no overlap */
       msLayerClose(layer);
       return NULL;
@@ -696,12 +703,9 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
 
       msClipPolylineRect(&shapegrid, cliprect);
      
-#ifdef USE_AGG
-      if(MS_RENDERER_AGG(map->outputformat))
-        msTransformShapeAGG(&shapegrid, map->extent, map->cellsize);
-#else
-      msTransformShapeToPixel(&shapegrid, map->extent, map->cellsize);
-#endif
+
+      msTransformShapeToPixelRound(&shapegrid, map->extent, map->cellsize);
+
      
  
       if(shapegrid.numlines <= 0 || shapegrid.line[0].numpoints < 2) { /* once clipped the shape didn't need to be drawn */
@@ -771,14 +775,14 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
                     continue;
 
                   if (shapegrid.text)
-                    pszLabel =  strdup(shapegrid.text);
+                    pszLabel =  msStrdup(shapegrid.text);
                   else
                   {
                       _FormatLabel(layer, &tmpshape, tmpshape.line[0].point[tmpshape.line[0].numpoints-1].x );
                       if (tmpshape.text)
-                        pszLabel = strdup(tmpshape.text);
+                        pszLabel = msStrdup(tmpshape.text);
                       else
-                        pszLabel = strdup("unknown");
+                        pszLabel = msStrdup("unknown");
                   }
                   /*validate that the  value is not already in the array*/
                   if ( psValues->nBottom > 0)
@@ -796,20 +800,20 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
                     }
                   if (psValues->pasBottom == NULL)
                     {
-                      psValues->pasBottom = (pointObj *)malloc(sizeof(pointObj));
-                      psValues->papszBottomLabels = (char **)malloc(sizeof(char *));
+                      psValues->pasBottom = (pointObj *)msSmallMalloc(sizeof(pointObj));
+                      psValues->papszBottomLabels = (char **)msSmallMalloc(sizeof(char *));
                       psValues->nBottom = 1;
                     }
                   else
                     {
                       psValues->nBottom++;
-                      psValues->pasBottom = (pointObj *)realloc(psValues->pasBottom, sizeof(pointObj)*psValues->nBottom);
-                      psValues->papszBottomLabels = (char **)realloc(psValues->papszBottomLabels, sizeof(char *)*psValues->nBottom);
+                      psValues->pasBottom = (pointObj *)msSmallRealloc(psValues->pasBottom, sizeof(pointObj)*psValues->nBottom);
+                      psValues->papszBottomLabels = (char **)msSmallRealloc(psValues->papszBottomLabels, sizeof(char *)*psValues->nBottom);
                     }
                       
                   psValues->pasBottom[psValues->nBottom-1].x = oFirstPoint.x;
                   psValues->pasBottom[psValues->nBottom-1].y = oFirstPoint.y;
-                  psValues->papszBottomLabels[psValues->nBottom-1] = strdup(pszLabel);
+                  psValues->papszBottomLabels[psValues->nBottom-1] = msStrdup(pszLabel);
 
                   msFree(pszLabel); 
 
@@ -825,14 +829,14 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
                     continue;
 
                   if (shapegrid.text)
-                    pszLabel =  strdup(shapegrid.text);
+                    pszLabel =  msStrdup(shapegrid.text);
                   else
                   {
                       _FormatLabel(layer, &tmpshape, tmpshape.line[0].point[tmpshape.line[0].numpoints-1].x );
                        if (tmpshape.text)
-                         pszLabel = strdup(tmpshape.text);
+                         pszLabel = msStrdup(tmpshape.text);
                        else
-                         pszLabel = strdup("unknown");
+                         pszLabel = msStrdup("unknown");
                   }
                   /*validate if same value is not already there*/
                   if ( psValues->nTop > 0)
@@ -853,20 +857,20 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
                     
                   if (psValues->pasTop == NULL)
                     {
-                      psValues->pasTop = (pointObj *)malloc(sizeof(pointObj));
-                      psValues->papszTopLabels = (char **)malloc(sizeof(char *));
+                      psValues->pasTop = (pointObj *)msSmallMalloc(sizeof(pointObj));
+                      psValues->papszTopLabels = (char **)msSmallMalloc(sizeof(char *));
                       psValues->nTop = 1;
                     }
                   else
                     {
                       psValues->nTop++;
-                      psValues->pasTop = (pointObj *)realloc(psValues->pasTop, sizeof(pointObj)*psValues->nTop);
-                      psValues->papszTopLabels = (char **)realloc(psValues->papszTopLabels, sizeof(char *)*psValues->nTop);
+                      psValues->pasTop = (pointObj *)msSmallRealloc(psValues->pasTop, sizeof(pointObj)*psValues->nTop);
+                      psValues->papszTopLabels = (char **)msSmallRealloc(psValues->papszTopLabels, sizeof(char *)*psValues->nTop);
                     }
                       
                   psValues->pasTop[psValues->nTop-1].x = oLastPoint.x;
                   psValues->pasTop[psValues->nTop-1].y = oLastPoint.y;
-                  psValues->papszTopLabels[psValues->nTop-1] = strdup(pszLabel);
+                  psValues->papszTopLabels[psValues->nTop-1] = msStrdup(pszLabel);
 
                   msFree(pszLabel); 
                 }
@@ -897,14 +901,14 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
                     continue;
 
                   if (shapegrid.text)
-                    pszLabel =  strdup(shapegrid.text);
+                    pszLabel =  msStrdup(shapegrid.text);
                   else
                   {
                       _FormatLabel(layer, &tmpshape, tmpshape.line[0].point[tmpshape.line[0].numpoints-1].y );
                        if (tmpshape.text)
-                         pszLabel = strdup(tmpshape.text);
+                         pszLabel = msStrdup(tmpshape.text);
                        else
-                         pszLabel = strdup("unknown");
+                         pszLabel = msStrdup("unknown");
                   }
 
                   /*validate that the previous value is not the same*/
@@ -923,20 +927,20 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
                     }
                   if (psValues->pasLeft == NULL)
                     {
-                      psValues->pasLeft = (pointObj *)malloc(sizeof(pointObj));
-                      psValues->papszLeftLabels = (char **)malloc(sizeof(char *));
+                      psValues->pasLeft = (pointObj *)msSmallMalloc(sizeof(pointObj));
+                      psValues->papszLeftLabels = (char **)msSmallMalloc(sizeof(char *));
                       psValues->nLeft = 1;
                     }
                   else
                     {
                       psValues->nLeft++;
-                      psValues->pasLeft = (pointObj *)realloc(psValues->pasLeft, sizeof(pointObj)*psValues->nLeft);
-                      psValues->papszLeftLabels = (char **)realloc(psValues->papszLeftLabels, sizeof(char *)*psValues->nLeft);
+                      psValues->pasLeft = (pointObj *)msSmallRealloc(psValues->pasLeft, sizeof(pointObj)*psValues->nLeft);
+                      psValues->papszLeftLabels = (char **)msSmallRealloc(psValues->papszLeftLabels, sizeof(char *)*psValues->nLeft);
                     }
                       
                   psValues->pasLeft[psValues->nLeft-1].x = oFirstPoint.x;
                   psValues->pasLeft[psValues->nLeft-1].y = oFirstPoint.y;
-                  psValues->papszLeftLabels[psValues->nLeft-1] = strdup(pszLabel);
+                  psValues->papszLeftLabels[psValues->nLeft-1] = msStrdup(pszLabel);
                   msFree(pszLabel); 
                 }
               /*first point should cross the RIGHT base where x=map=>width axis*/
@@ -950,14 +954,14 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
                     continue;
 
                   if (shapegrid.text)
-                    pszLabel =  strdup(shapegrid.text);
+                    pszLabel =  msStrdup(shapegrid.text);
                   else
                   {
                       _FormatLabel(layer, &tmpshape, tmpshape.line[0].point[tmpshape.line[0].numpoints-1].y );
                        if (tmpshape.text)
-                         pszLabel = strdup(tmpshape.text);
+                         pszLabel = msStrdup(tmpshape.text);
                        else
-                         pszLabel = strdup("unknown");
+                         pszLabel = msStrdup("unknown");
                   }
 
                   /*validate that the previous value is not the same*/
@@ -975,20 +979,20 @@ graticuleIntersectionObj *msGraticuleLayerGetIntersectionPoints(mapObj *map,
                     }
                   if (psValues->pasRight == NULL)
                     {
-                      psValues->pasRight = (pointObj *)malloc(sizeof(pointObj));
-                      psValues->papszRightLabels = (char **)malloc(sizeof(char *));
+                      psValues->pasRight = (pointObj *)msSmallMalloc(sizeof(pointObj));
+                      psValues->papszRightLabels = (char **)msSmallMalloc(sizeof(char *));
                       psValues->nRight = 1;
                     }
                   else
                     {
                       psValues->nRight++;
-                      psValues->pasRight = (pointObj *)realloc(psValues->pasRight, sizeof(pointObj)*psValues->nRight);
-                      psValues->papszRightLabels = (char **)realloc(psValues->papszRightLabels, sizeof(char *)*psValues->nRight);
+                      psValues->pasRight = (pointObj *)msSmallRealloc(psValues->pasRight, sizeof(pointObj)*psValues->nRight);
+                      psValues->papszRightLabels = (char **)msSmallRealloc(psValues->papszRightLabels, sizeof(char *)*psValues->nRight);
                     }
                       
                   psValues->pasRight[psValues->nRight-1].x = oLastPoint.x;
                   psValues->pasRight[psValues->nRight-1].y = oLastPoint.y;
-                  psValues->papszRightLabels[psValues->nRight-1] = strdup(pszLabel);
+                  psValues->papszRightLabels[psValues->nRight-1] = msStrdup(pszLabel);
 
                   msFree(pszLabel); 
                 }
@@ -1067,7 +1071,7 @@ static void _FormatLabel( layerObj *pLayer, shapeObj *pShape, double dDataToForm
     sprintf( cBuffer, pInfo->labelformat, dDataToFormat );
   }
    
-  pShape->text = strdup( cBuffer );
+  pShape->text = msStrdup( cBuffer );
 }
 
 /**********************************************************************************************************************
@@ -1079,6 +1083,7 @@ static int _AdjustLabelPosition( layerObj *pLayer, shapeObj *pShape, msGraticule
   graticuleObj *pInfo = (graticuleObj  *) pLayer->layerinfo;
   rectObj rectLabel;
   pointObj ptPoint;
+  double size = pLayer->class[0]->label.size; //TODO TBT: adjust minsize/maxsize/resolution
 
   if( pInfo == NULL || pShape == NULL ) {
     msSetError(MS_MISCERR, "Assertion failed: Null shape or layerinfo!, ", "_AdjustLabelPosition()");
@@ -1095,11 +1100,12 @@ static int _AdjustLabelPosition( layerObj *pLayer, shapeObj *pShape, msGraticule
     msProjectShape( &pLayer->projection, &pLayer->map->projection, pShape );
 #endif
 
-  if(pLayer->transform) 
-    msTransformShapeToPixel(pShape, pLayer->map->extent, pLayer->map->cellsize);
+  if(pLayer->transform) {
+     msTransformShapeToPixelRound(pShape, pLayer->map->extent, pLayer->map->cellsize);
+  }
 
-  if(msGetLabelSize(NULL, pShape->text, &pLayer->class[0]->label, &rectLabel, &pLayer->map->fontset, 1.0, MS_FALSE,NULL) != 0)
-    return MS_FAILURE;  /* msSetError already called */
+  if(msGetLabelSize(pLayer->map, &pLayer->class[0]->label,pShape->text, size,&rectLabel,NULL) != MS_SUCCESS)
+	  return MS_FAILURE;  /* msSetError already called */
 
   switch( ePosition ) {
   case posBottom:
