@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maplegend.c 10015 2010-03-25 15:49:05Z assefa $
+ * $Id: maplegend.c 11673 2011-05-11 16:33:28Z tbonfort $
  *
  * Project:  MapServer
  * Purpose:  Legend generation.
@@ -29,7 +29,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: maplegend.c 10015 2010-03-25 15:49:05Z assefa $")
+MS_CVSID("$Id: maplegend.c 11673 2011-05-11 16:33:28Z tbonfort $")
 
 #define PSF .8
 #define VMARGIN 5 /* margin at top and bottom of legend graphic */
@@ -44,31 +44,74 @@ int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *theclass,
         int width, int height, imageObj *image, int dstX, int dstY)
 {
   int i, type;
+  double offset;
   shapeObj box, zigzag;
   pointObj marker;
   char szPath[MS_MAXPATHLEN];
   styleObj outline_style;
-  
-  if(MS_RENDERER_GD(map->outputformat))
-    /* keep GD specific code here for now as it supports clipping */
-	if(MS_VALID_COLOR(map->legend.outlinecolor))
-	  gdImageSetClip(image->img.gd, dstX, dstY, dstX + width - 1, dstY + height - 1);
-  
+  imageObj *image_draw = image;
+  int originalopacity = lp->opacity;
+  rendererVTableObj *renderer;
+  outputFormatObj *transFormat = NULL, *altFormat=NULL;
+  const char *alternativeFormatString = NULL;
 
+  if(!MS_RENDERER_PLUGIN(image->format)) {
+      msSetError(MS_MISCERR,"unsupported image format","msDrawLegendIcon()");
+      return MS_FAILURE;
+  }
+ 
+  alternativeFormatString = msLayerGetProcessingKey(lp, "RENDERER");
+  if (MS_RENDERER_PLUGIN(image_draw->format) && alternativeFormatString!=NULL && 
+      (altFormat=  msSelectOutputFormat(map, alternativeFormatString)))
+  {
+      msInitializeRendererVTable(altFormat);
+
+      image_draw = msImageCreate(image->width, image->height,
+                                 altFormat, image->imagepath, image->imageurl, map->resolution, map->defresolution, &map->imagecolor);
+      renderer = MS_IMAGE_RENDERER(image_draw);
+  }
+  else {
+      renderer = MS_IMAGE_RENDERER(image_draw);
+      if (lp->opacity > 0 && lp->opacity < 100) {
+         if (!renderer->supports_transparent_layers) {
+              image_draw = msImageCreate(image->width, image->height,
+                  image->format, image->imagepath, image->imageurl, map->resolution, map->defresolution, NULL);
+              if (!image_draw) {
+                  msSetError(MS_MISCERR, "Unable to initialize temporary transparent image.",
+                             "msDrawLegendIcon()");
+                  return (MS_FAILURE);
+              }
+              /* set opacity to full, as the renderer should be rendering a fully opaque image */
+              lp->opacity=100;
+          } 
+      }
+  }
+  
+  
+  if(renderer->supports_clipping && MS_VALID_COLOR(map->legend.outlinecolor)) {
+    /* keep GD specific code here for now as it supports clipping */
+    rectObj clip;
+    clip.maxx = dstX + width - 1;
+    clip.maxy = dstY + height -1;
+    clip.minx = dstX;
+    clip.miny = dstY;
+    renderer->setClip(image_draw,clip);
+  }
+  
   /* initialize the box used for polygons and for outlines */
-  box.line = (lineObj *)malloc(sizeof(lineObj));
+  box.line = (lineObj *)msSmallMalloc(sizeof(lineObj));
   box.numlines = 1;
-  box.line[0].point = (pointObj *)malloc(sizeof(pointObj)*5);
+  box.line[0].point = (pointObj *)msSmallMalloc(sizeof(pointObj)*5);
   box.line[0].numpoints = 5;
 
-  box.line[0].point[0].x = dstX;
-  box.line[0].point[0].y = dstY;
-  box.line[0].point[1].x = dstX + width - 1;
-  box.line[0].point[1].y = dstY;
-  box.line[0].point[2].x = dstX + width - 1;
-  box.line[0].point[2].y = dstY + height - 1;
-  box.line[0].point[3].x = dstX;
-  box.line[0].point[3].y = dstY + height - 1;
+  box.line[0].point[0].x = dstX + 0.5;
+  box.line[0].point[0].y = dstY + 0.5;
+  box.line[0].point[1].x = dstX + width - 0.5;
+  box.line[0].point[1].y = dstY + 0.5;
+  box.line[0].point[2].x = dstX + width - 0.5;
+  box.line[0].point[2].y = dstY + height - 0.5;
+  box.line[0].point[3].x = dstX + 0.5;
+  box.line[0].point[3].y = dstY + height - 0.5;
   box.line[0].point[4].x = box.line[0].point[0].x;
   box.line[0].point[4].y = box.line[0].point[0].y;
   box.line[0].numpoints = 5;
@@ -100,7 +143,7 @@ int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *theclass,
     marker.y = dstY + MS_NINT(height / 2.0);
     if (theclass->numstyles > 0) {
       for(i=0; i<theclass->numstyles; i++)
-        msDrawMarkerSymbol(&map->symbolset, image, &marker, theclass->styles[i], lp->scalefactor);          
+        msDrawMarkerSymbol(&map->symbolset, image_draw, &marker, theclass->styles[i], lp->scalefactor);          
     } else if (theclass->label.size!=-1) {
       labelObj label = theclass->label;
       double lsize = label.size;
@@ -109,7 +152,7 @@ int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *theclass,
       label.angle = 0;
       label.position = MS_CC;
       if (label.type == MS_TRUETYPE) label.size = height;
-      msDrawLabel(map, image, marker, (char*)"Az", &label,1.0);
+      msDrawLabel(map, image_draw, marker, (char*)"Az", &label,1.0);
       label.size = lsize;
       label.position = lpos;
       label.angle = langle;
@@ -131,7 +174,7 @@ int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *theclass,
       symbol = map->symbolset.symbol[symbolNum];
       
       initStyle(&imgStyle);
-      /*set size so thet symbol will be scaled properly #3296*/
+      /*set size so that symbol will be scaled properly #3296*/
       if (width/symbol->sizex < height/symbol->sizey)
         imgStyle.size = symbol->sizey*(width/symbol->sizex);
       else
@@ -141,30 +184,35 @@ int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *theclass,
         imgStyle.maxsize = imgStyle.size;
 
       imgStyle.symbol = symbolNum;
-      msDrawMarkerSymbol(&map->symbolset,image,&marker,&imgStyle,lp->scalefactor);
+      msDrawMarkerSymbol(&map->symbolset,image_draw,&marker,&imgStyle,lp->scalefactor);
       /* TO DO: we may want to handle this differently depending on the relative size of the keyimage */
     } else {
       for(i=0; i<theclass->numstyles; i++)
-        msDrawMarkerSymbol(&map->symbolset, image, &marker, theclass->styles[i], lp->scalefactor);
+        msDrawMarkerSymbol(&map->symbolset, image_draw, &marker, theclass->styles[i], lp->scalefactor);
     }
     break;
   case MS_LAYER_LINE:
-    zigzag.line = (lineObj *)malloc(sizeof(lineObj));
+    offset = 1;
+    /* To set the offset, we only check the size/width parameter of the first style */
+    if (theclass->numstyles > 0) {
+      offset = (theclass->styles[0]->size != -1) ? theclass->styles[0]->size/2 : theclass->styles[0]->width/2;
+    }
+    zigzag.line = (lineObj *)msSmallMalloc(sizeof(lineObj));
     zigzag.numlines = 1;
-    zigzag.line[0].point = (pointObj *)malloc(sizeof(pointObj)*4);
+    zigzag.line[0].point = (pointObj *)msSmallMalloc(sizeof(pointObj)*4);
     zigzag.line[0].numpoints = 4;
 
-    zigzag.line[0].point[0].x = dstX;
-    zigzag.line[0].point[0].y = dstY + height - 1;
+    zigzag.line[0].point[0].x = dstX + offset;
+    zigzag.line[0].point[0].y = dstY + height - offset;
     zigzag.line[0].point[1].x = dstX + MS_NINT(width / 3.0) - 1; 
-    zigzag.line[0].point[1].y = dstY;
+    zigzag.line[0].point[1].y = dstY + offset;
     zigzag.line[0].point[2].x = dstX + MS_NINT(2.0 * width / 3.0) - 1; 
-    zigzag.line[0].point[2].y = dstY + height - 1; 
-    zigzag.line[0].point[3].x = dstX + width - 1; 
-    zigzag.line[0].point[3].y = dstY; 
+    zigzag.line[0].point[2].y = dstY + height - offset; 
+    zigzag.line[0].point[3].x = dstX + width - offset; 
+    zigzag.line[0].point[3].y = dstY + offset;
 
     for(i=0; i<theclass->numstyles; i++)
-      msDrawLineSymbol(&map->symbolset, image, &zigzag, theclass->styles[i], lp->scalefactor); 
+      msDrawLineSymbol(&map->symbolset, image_draw, &zigzag, theclass->styles[i], lp->scalefactor); 
 
     free(zigzag.line[0].point);
     free(zigzag.line);    
@@ -174,7 +222,7 @@ int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *theclass,
   case MS_LAYER_CHART:
   case MS_LAYER_POLYGON:
     for(i=0; i<theclass->numstyles; i++)     
-      msDrawShadeSymbol(&map->symbolset, image, &box, theclass->styles[i], lp->scalefactor);
+      msDrawShadeSymbol(&map->symbolset, image_draw, &box, theclass->styles[i], lp->scalefactor);
     break;
   default:
     return MS_FAILURE;
@@ -185,10 +233,53 @@ int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *theclass,
   if(MS_VALID_COLOR(map->legend.outlinecolor)) {
     initStyle(&outline_style);
     outline_style.color = map->legend.outlinecolor;
-    msDrawLineSymbol(&map->symbolset, image, &box, &outline_style, 1.0);
+    msDrawLineSymbol(&map->symbolset, image_draw, &box, &outline_style, 1.0);
     /* reset clipping rectangle */
-    if(MS_RENDERER_GD(map->outputformat))
-    	gdImageSetClip(image->img.gd, 0, 0, image->width - 1, image->height - 1);
+    if(renderer->supports_clipping)
+    	renderer->resetClip(image_draw);
+  }
+
+  if (altFormat)
+  {
+      rendererVTableObj *renderer = MS_IMAGE_RENDERER(image);
+      rendererVTableObj *altrenderer = MS_IMAGE_RENDERER(image_draw);
+      rasterBufferObj rb;
+      memset(&rb,0,sizeof(rasterBufferObj));
+
+      altrenderer->getRasterBufferHandle(image_draw,&rb);
+      renderer->mergeRasterBuffer(image,&rb,lp->opacity*0.01,0,0,0,0,rb.width,rb.height);  
+      /* 
+       * hack to work around bug #3834: if we have use an alternate renderer, the symbolset may contain
+       * symbols that reference it. We want to remove those references before the altFormat is destroyed
+       * to avoid a segfault and/or a leak, and so the the main renderer doesn't pick the cache up thinking
+       * it's for him.
+       */
+      for(i=0; i<map->symbolset.numsymbols; i++) {
+         if (map->symbolset.symbol[i]!=NULL) {
+            symbolObj *s = map->symbolset.symbol[i];
+            if(s->renderer == altrenderer) {
+	            altrenderer->freeSymbol(s);
+               s->renderer = NULL;
+            }
+         }
+      }
+      msFreeImage(image_draw);
+      
+  }
+  else if(image != image_draw) {
+	  rendererVTableObj *renderer = MS_IMAGE_RENDERER(image_draw);
+	  rasterBufferObj rb;
+	  memset(&rb,0,sizeof(rasterBufferObj));
+
+	  lp->opacity = originalopacity;
+
+	  renderer->getRasterBufferHandle(image_draw,&rb);
+	  renderer->mergeRasterBuffer(image,&rb,lp->opacity*0.01,0,0,0,0,rb.width,rb.height);  
+	  msFreeImage(image_draw);
+
+	  /* deref and possibly free temporary transparent output format.  */
+	  msApplyOutputFormat( &transFormat, NULL, MS_NOOVERRIDE, MS_NOOVERRIDE, MS_NOOVERRIDE );
+
   }
 
   free(box.line[0].point);
@@ -203,23 +294,20 @@ imageObj *msCreateLegendIcon(mapObj* map, layerObj* lp, classObj* class, int wid
   imageObj *image;
   outputFormatObj *format = NULL;
   int i = 0;
-
-  if(!map->outputformat || (!MS_RENDERER_GD(map->outputformat) && !MS_RENDERER_AGG(map->outputformat) )) {
-    msSetError(MS_GDERR, "Map outputformat must be set to a GD or AGG format!", "msCreateLegendIcon()");
+  
+  rendererVTableObj *renderer = MS_MAP_RENDERER(map);
+  
+  if( !renderer ) {
+    msSetError(MS_MISCERR, "invalid map outputformat", "msCreateLegendIcon()");
     return(NULL);
   }
 
   /* ensure we have an image format representing the options for the legend */
   msApplyOutputFormat(&format, map->outputformat, map->legend.transparent, map->legend.interlace, MS_NOOVERRIDE);
-
-  /* create image */
-#ifdef USE_AGG
-  if(MS_RENDERER_AGG(map->outputformat))
-     image = msImageCreateAGG(width, height, map->outputformat, map->web.imagepath, map->web.imageurl, map->resolution, map->defresolution);        
-  else
-#endif
-    image = msImageCreateGD(width, height, map->outputformat, map->web.imagepath, map->web.imageurl, map->resolution, map->defresolution);
-
+  
+  image = msImageCreate(width,height,format,map->web.imagepath, map->web.imageurl,
+		  map->resolution, map->defresolution, &(map->legend.imagecolor));
+  
   /* drop this reference to output format */
   msApplyOutputFormat( &format, NULL, MS_NOOVERRIDE, MS_NOOVERRIDE, MS_NOOVERRIDE );
 
@@ -228,14 +316,6 @@ imageObj *msCreateLegendIcon(mapObj* map, layerObj* lp, classObj* class, int wid
     return(NULL);
   }
 
-  /* allocate the background color */
-#ifdef USE_AGG
-  if(MS_RENDERER_AGG(map->outputformat))
-    msImageInitAGG( image, &(map->legend.imagecolor));
-  else
-#endif
-    msImageInitGD( image, &(map->legend.imagecolor));
-
   /* Call drawLegendIcon with destination (0, 0) */
   /* Return an empty image if lp==NULL || class=NULL  */
   /* (If class is NULL draw the legend for all classes. Modifications done */
@@ -250,10 +330,6 @@ imageObj *msCreateLegendIcon(mapObj* map, layerObj* lp, classObj* class, int wid
       }
     }
   }
-#ifdef USE_AGG
-  if(MS_RENDERER_AGG(map->outputformat))
-    msAlphaAGG2GD(image);
-#endif
   return image;
 }
 
@@ -267,49 +343,54 @@ imageObj *msCreateLegendIcon(mapObj* map, layerObj* lp, classObj* class, int wid
  *   MS_SUCCESS
  *   MS_FAILURE
  */
-int msLegendCalcSize(mapObj *map, int scale_independent, int *size_x, int *size_y, layerObj *psForLayer) 
+int msLegendCalcSize(mapObj *map, int scale_independent, int *size_x, int *size_y, 
+                     int *layer_index, int num_layers)
 {
   int i, j;
   int status, maxwidth=0, nLegendItems=0;
-  char *transformedText; /* Label text after applying wrapping, encoding if necessary */
+  char *text, *transformedText; /* legend text before/after applying wrapping, encoding if necessary */
   layerObj *lp;  
   rectObj rect;
-    
-  /* Reset sizes */
+  int current_layers=0;
+
+  /* reset sizes */
   *size_x = 0;
   *size_y = 0;
     
-  /* Enable scale-dependent calculations */
+  /* enable scale-dependent calculations */
   if(!scale_independent) {
     map->cellsize = msAdjustExtent(&(map->extent), map->width, map->height);
     status = msCalculateScale(map->extent, map->units, map->width, map->height, map->resolution, &map->scaledenom);
     if(status != MS_SUCCESS) return MS_FAILURE;
   }
 
-
   /*
    * step through all map classes, and for each one that will be displayed
    * calculate the label size
    */
-  for(i=0; i<map->numlayers; i++) {
-    if(psForLayer) {
-      lp = psForLayer;
-      i = map->numlayers;
-    } else
+  if (layer_index != NULL && num_layers >0)
+    current_layers  = num_layers;
+  else
+    current_layers = map->numlayers;
+       
+  for(i=0; i< current_layers; i++) {
+
+    if (layer_index != NULL && num_layers > 0)
+      lp = GET_LAYER(map, layer_index[i]);
+    else
       lp = (GET_LAYER(map, map->layerorder[i]));
 
-    if((lp->status == MS_OFF && lp != psForLayer) || (lp->type == MS_LAYER_QUERY)) /* skip it */
+    if((lp->status == MS_OFF && (layer_index == NULL || num_layers <= 0)) || (lp->type == MS_LAYER_QUERY)) /* skip it */
       continue;
             
     if(!scale_independent && map->scaledenom > 0) {
-      if((lp->maxscaledenom > 0) && (map->scaledenom > lp->maxscaledenom))
-        continue;
-      if((lp->minscaledenom > 0) && (map->scaledenom <= lp->minscaledenom))
-        continue;
+      if((lp->maxscaledenom > 0) && (map->scaledenom > lp->maxscaledenom)) continue;
+      if((lp->minscaledenom > 0) && (map->scaledenom <= lp->minscaledenom)) continue;
     }
         
     for(j=lp->numclasses-1; j>=0; j--) {
-      if(!lp->class[j]->name) continue; /* skip it */
+      text = lp->class[j]->title?lp->class[j]->title:lp->class[j]->name; /* point to the right legend text, title takes precedence */
+      if(!text) continue; /* skip it */
             
       /* skip the class if the classgroup is defined */
       if(lp->classgroup && (lp->class[j]->group == NULL || strcasecmp(lp->class[j]->group, lp->classgroup) != 0))
@@ -317,11 +398,8 @@ int msLegendCalcSize(mapObj *map, int scale_independent, int *size_x, int *size_
 
        /* verify class scale */
        if(!scale_independent && map->scaledenom > 0) {
-         if((lp->class[j]->maxscaledenom > 0) && (map->scaledenom > lp->class[j]->maxscaledenom))
-           continue;
-                    
-         if((lp->class[j]->minscaledenom > 0) && (map->scaledenom <= lp->class[j]->minscaledenom))
-           continue;
+         if((lp->class[j]->maxscaledenom > 0) && (map->scaledenom > lp->class[j]->maxscaledenom)) continue;                    
+         if((lp->class[j]->minscaledenom > 0) && (map->scaledenom <= lp->class[j]->minscaledenom)) continue;
        }
             
        /*
@@ -333,11 +411,11 @@ int msLegendCalcSize(mapObj *map, int scale_independent, int *size_x, int *size_
         * same as the class name pointer
         */
        if(map->legend.label.encoding || map->legend.label.wrap)
-         transformedText = msTransformLabelText(map,NULL,&map->legend.label, lp->class[j]->name);
+         transformedText = msTransformLabelText(map,NULL,&map->legend.label, text);
        else
-         transformedText = strdup(lp->class[j]->name);
+         transformedText = msStrdup(text);
 
-       if(transformedText == NULL || msGetLabelSize(NULL,transformedText, &map->legend.label, &rect, &(map->fontset), 1.0, MS_FALSE,NULL) != 0) { /* something bad happened */
+       if(transformedText == NULL || msGetLabelSize(map, &map->legend.label, transformedText, map->legend.label.size, &rect, NULL) != MS_SUCCESS) { /* something bad happened */
          if(transformedText) msFree(transformedText);
          return MS_FAILURE;
        }
@@ -373,7 +451,6 @@ int msLegendCalcSize(mapObj *map, int scale_independent, int *size_x, int *size_
 */
 imageObj *msDrawLegend(mapObj *map, int scale_independent)
 {
-  gdImagePtr img; /* image data structure */
   int i,j; /* loop counters */
   pointObj pnt;
   int size_x, size_y=0;
@@ -381,6 +458,8 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
   rectObj rect;
   imageObj *image = NULL;
   outputFormatObj *format = NULL;
+  char *text;
+
   struct legend_struct {
     int height;
     char *transformedText;
@@ -391,13 +470,12 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
   typedef struct legend_struct legendlabel;
   legendlabel *head=NULL,*cur=NULL;
   
-  if(MS_RENDERER_PLUGIN(map->outputformat)) {
-    msSetError(MS_MISCERR, "Scalebar not supported yet", "msDrawScalebar()");
-    return(NULL);
+  if(!MS_RENDERER_PLUGIN(map->outputformat)) {
+      msSetError(MS_MISCERR,"unsupported output format","msDrawLegend()");
+      return NULL;
   }
-  
   if(msValidateContexts(map) != MS_SUCCESS) return NULL; /* make sure there are no recursive REQUIRES or LABELREQUIRES expressions */
-  if(msLegendCalcSize(map, scale_independent, &size_x, &size_y, NULL) != MS_SUCCESS) return NULL;
+  if(msLegendCalcSize(map, scale_independent, &size_x, &size_y, NULL, 0) != MS_SUCCESS) return NULL;
 
   /*
    * step through all map classes, and for each one that will be displayed
@@ -414,45 +492,48 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
       if((lp->minscaledenom > 0) && (map->scaledenom <= lp->minscaledenom)) continue;
     }
 
-    if (!scale_independent && lp->maxscaledenom <=0 && lp->minscaledenom <=0) {
+    if(!scale_independent && lp->maxscaledenom <=0 && lp->minscaledenom <=0) {
       if((lp->maxgeowidth > 0) && ((map->extent.maxx - map->extent.minx) > lp->maxgeowidth)) continue;
       if((lp->mingeowidth > 0) && ((map->extent.maxx - map->extent.minx) < lp->mingeowidth)) continue;
     }
 
     for(j=lp->numclasses-1;j>=0;j--) {
+      text = lp->class[j]->title?lp->class[j]->title:lp->class[j]->name; /* point to the right legend text, title takes precedence */
+      if(!text) continue; /* skip it */
 
       /* skip the class if the classgroup is defined */
       if(lp->classgroup && (lp->class[j]->group == NULL || strcasecmp(lp->class[j]->group, lp->classgroup) != 0))
         continue;
-      if(!lp->class[j]->name) continue; /* skip it */
+
       if(!scale_independent && map->scaledenom > 0) {  /* verify class scale here */
         if((lp->class[j]->maxscaledenom > 0) && (map->scaledenom > lp->class[j]->maxscaledenom)) continue;
         if((lp->class[j]->minscaledenom > 0) && (map->scaledenom <= lp->class[j]->minscaledenom)) continue;
       }
-      cur = (legendlabel*) malloc(sizeof(legendlabel));
+
+      cur = (legendlabel*) msSmallMalloc(sizeof(legendlabel));
             
       /*
        * apply encoding and line wrapping to the legend label if requested
        * this is done conditionnally as the text transformation function
        * does some memory allocations that can be avoided in most cases.
        * the transformed text must be freed once finished, this must be done
-       * conditionnally by testing if the transformed text pointer is the
+       * conditionally by testing if the transformed text pointer is the
        * same as the class name pointer
        */
       if(map->legend.label.encoding || map->legend.label.wrap)
-        cur->transformedText = msTransformLabelText(map,NULL,&map->legend.label,lp->class[j]->name);
+        cur->transformedText = msTransformLabelText(map,NULL,&map->legend.label,text);
       else
-        cur->transformedText = lp->class[j]->name;
+        cur->transformedText = msStrdup(text); /* so we can always do msFree() when cleaning up */
 
       cur->theclass = lp->class[j];
       cur->layer = lp;
       cur->pred = head;
       head = cur;
 
-      if(cur->transformedText==NULL || msGetLabelSize(NULL,cur->transformedText, &map->legend.label, &rect, &(map->fontset), 1.0, MS_FALSE,NULL) != 0) { /* something bad happened, free allocated mem */
-        while(cur) {
-          if(cur->transformedText!=cur->theclass->name)
-            free(cur->transformedText);
+      if(cur->transformedText==NULL || 
+    		  msGetLabelSize(map, &map->legend.label, cur->transformedText, map->legend.label.size, &rect, NULL) != MS_SUCCESS) { /* something bad happened, free allocated mem */
+        while(cur) {          
+          free(cur->transformedText);
           head = cur;
           cur = cur->pred;
           free(head);
@@ -468,32 +549,16 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
   msApplyOutputFormat(&format, map->outputformat, map->legend.transparent, map->legend.interlace, MS_NOOVERRIDE);
 
   /* initialize the legend image */
-#ifdef USE_AGG
-  if(MS_RENDERER_AGG(map->outputformat))
-    image = msImageCreateAGG(size_x, size_y, format, map->web.imagepath, map->web.imageurl, map->resolution, map->defresolution);        
-  else
-#endif
-    image = msImageCreateGD(size_x, size_y, format, map->web.imagepath, map->web.imageurl, map->resolution, map->defresolution);
+  image = msImageCreate(size_x, size_y, format, map->web.imagepath, map->web.imageurl, map->resolution, map->defresolution, &map->legend.imagecolor);
+  if(!image) {
+    msSetError(MS_MISCERR, "Unable to initialize image.", "msDrawLegend()");
+    return NULL;
+  }
+  //image = renderer->createImage(size_x,size_y,format,&(map->legend.imagecolor));
 
   /* drop this reference to output format */
   msApplyOutputFormat(&format, NULL, MS_NOOVERRIDE, MS_NOOVERRIDE, MS_NOOVERRIDE);
 
-  if(image)
-    img = image->img.gd;
-  else {
-    msSetError(MS_GDERR, "Unable to initialize image.", "msDrawLegend()");
-    return(NULL);
-  }
-
-  /* Set background */
-#ifdef USE_AGG
-  if(MS_RENDERER_AGG(map->outputformat))
-    msImageInitAGG( image, &(map->legend.imagecolor));
-  else
-#endif
-    msImageInitGD(image, &(map->legend.imagecolor));
-
-
   msClearPenValues(map); /* just in case the mapfile has already been processed */
   pnt.y = VMARGIN;
   pnt.x = HMARGIN + map->legend.keysizex + map->legend.keyspacingx;
@@ -502,8 +567,7 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
     int number_of_newlines=0, offset=0;
 
     /* set the scale factor so that scale dependant symbols are drawn in the legend with their default size */
-    if(cur->layer->sizeunits != MS_PIXELS)
-    {
+    if(cur->layer->sizeunits != MS_PIXELS) {
        map->cellsize = msAdjustExtent(&(map->extent), map->width, map->height);
       cur->layer->scalefactor = (msInchesPerUnit(cur->layer->sizeunits,0)/msInchesPerUnit(map->units,0)) / map->cellsize;
     }
@@ -532,18 +596,12 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
     }
     pnt.y += map->legend.keyspacingy; /* bump y for next label */
         
-    /* free the transformed text if needed */
-    if(cur->transformedText!=cur->theclass->name)
-      free(cur->transformedText);
+    /* clean up */
+    free(cur->transformedText);
     head = cur;
     cur = cur->pred;
     free(head);
   } /* next legend */
-
-#ifdef USE_AGG
-  if(MS_RENDERER_AGG(map->outputformat))
-    msAlphaAGG2GD(image);
-#endif
     
   return(image);
 }
@@ -554,11 +612,15 @@ int msEmbedLegend(mapObj *map, imageObj *img)
   int s,l;
   pointObj point;
   imageObj *image = NULL;
+  symbolObj *legendSymbol;
 
-  if(MS_RENDERER_PLUGIN(map->outputformat)) {
-    msSetError(MS_MISCERR, "Scalebar not supported yet", "msDrawScalebar()");
+  rendererVTableObj *renderer;
+
+  if(!MS_RENDERER_PLUGIN(map->outputformat) || !MS_MAP_RENDERER(map)->supports_pixel_buffer) {
+    msSetError(MS_MISCERR, "unsupported output format", "msEmbedLegend()");
     return MS_FAILURE;
   }
+  renderer = MS_MAP_RENDERER(map);
 
   s = msGetSymbolIndex(&(map->symbolset), "legend", MS_FALSE);
   if(s != -1) 
@@ -567,57 +629,59 @@ int msEmbedLegend(mapObj *map, imageObj *img)
   if(msGrowSymbolSet(&map->symbolset) == NULL)
     return -1;
   s = map->symbolset.numsymbols;
+  legendSymbol = map->symbolset.symbol[s];
   map->symbolset.numsymbols++;
-  initSymbol(map->symbolset.symbol[s]);
+  initSymbol(legendSymbol);
 
-#ifdef USE_AGG
-  if(MS_RENDERER_AGG(map->outputformat))
-    msAlphaGD2AGG(img);
-#endif
-  
   /* render the legend. */
   image = msDrawLegend(map, MS_FALSE);
+  if( image == NULL ) return -1;
+
+  /* copy renderered legend image into symbol */
+  legendSymbol->pixmap_buffer = calloc(1,sizeof(rasterBufferObj));
+  MS_CHECK_ALLOC(legendSymbol->pixmap_buffer, sizeof(rasterBufferObj), MS_FAILURE);
+
+  if(MS_SUCCESS != renderer->getRasterBufferCopy(image,legendSymbol->pixmap_buffer))
+	  return MS_FAILURE;
+  legendSymbol->renderer = renderer;
 
-  /* steal the gdImage and free the rest of the imageObj */
-  map->symbolset.symbol[s]->img = image->img.gd; 
-  image->img.gd = NULL;
   msFreeImage( image );
 
-  if(!map->symbolset.symbol[s]->img) return(-1); /* something went wrong creating scalebar */
+  if(!legendSymbol->pixmap_buffer) return(-1); /* something went wrong creating scalebar */
 
-  map->symbolset.symbol[s]->type = MS_SYMBOL_PIXMAP; /* intialize a few things */
-  map->symbolset.symbol[s]->name = strdup("legend");  
-  map->symbolset.symbol[s]->sizex = map->symbolset.symbol[s]->img->sx;
-  map->symbolset.symbol[s]->sizey = map->symbolset.symbol[s]->img->sy;
+  legendSymbol->type = MS_SYMBOL_PIXMAP; /* intialize a few things */
+  legendSymbol->name = msStrdup("legend");  
+  legendSymbol->sizex = legendSymbol->pixmap_buffer->width;
+  legendSymbol->sizey = legendSymbol->pixmap_buffer->height;
 
   /* I'm not too sure this test is sufficient ... NFW. */
-  if(map->legend.transparent == MS_ON)
-    gdImageColorTransparent(map->symbolset.symbol[s]->img, 0);
+  //if(map->legend.transparent == MS_ON)
+  //  gdImageColorTransparent(legendSymbol->img_deprecated, 0);
 
   switch(map->legend.position) {
   case(MS_LL):
-    point.x = MS_NINT(map->symbolset.symbol[s]->img->sx/2.0);
-    point.y = map->height - MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
+    point.x = MS_NINT(legendSymbol->sizex/2.0);
+    point.y = map->height - MS_NINT(legendSymbol->sizey/2.0);
     break;
   case(MS_LR):
-    point.x = map->width - MS_NINT(map->symbolset.symbol[s]->img->sx/2.0);
-    point.y = map->height - MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
+    point.x = map->width - MS_NINT(legendSymbol->sizex/2.0);
+    point.y = map->height - MS_NINT(legendSymbol->sizey/2.0);
     break;
   case(MS_LC):
     point.x = MS_NINT(map->width/2.0);
-    point.y = map->height - MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
+    point.y = map->height - MS_NINT(legendSymbol->sizey/2.0);
     break;
   case(MS_UR):
-    point.x = map->width - MS_NINT(map->symbolset.symbol[s]->img->sx/2.0);
-    point.y = MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
+    point.x = map->width - MS_NINT(legendSymbol->sizex/2.0);
+    point.y = MS_NINT(legendSymbol->sizey/2.0);
     break;
   case(MS_UL):
-    point.x = MS_NINT(map->symbolset.symbol[s]->img->sx/2.0);
-    point.y = MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
+    point.x = MS_NINT(legendSymbol->sizex/2.0);
+    point.y = MS_NINT(legendSymbol->sizey/2.0);
     break;
   case(MS_UC):
     point.x = MS_NINT(map->width/2.0);
-    point.y = MS_NINT(map->symbolset.symbol[s]->img->sy/2.0);
+    point.y = MS_NINT(legendSymbol->sizey/2.0);
     break;
   }
 
@@ -628,7 +692,7 @@ int msEmbedLegend(mapObj *map, imageObj *img)
     l = map->numlayers;
     map->numlayers++;
     if(initLayer((GET_LAYER(map, l)), map) == -1) return(-1);
-    GET_LAYER(map, l)->name = strdup("__embed__legend");
+    GET_LAYER(map, l)->name = msStrdup("__embed__legend");
     GET_LAYER(map, l)->type = MS_LAYER_ANNOTATION;
 
     if(msGrowLayerClasses( GET_LAYER(map, l) ) == NULL)
@@ -642,7 +706,7 @@ int msEmbedLegend(mapObj *map, imageObj *img)
 
   GET_LAYER(map, l)->status = MS_ON;
 
-  if(msMaybeAllocateStyle(GET_LAYER(map, l)->class[0], 0)==MS_FAILURE) return MS_FAILURE;
+  if(msMaybeAllocateClassStyle(GET_LAYER(map, l)->class[0], 0)==MS_FAILURE) return MS_FAILURE;
   GET_LAYER(map, l)->class[0]->styles[0]->symbol = s;
   GET_LAYER(map, l)->class[0]->styles[0]->color.pen = -1;
   GET_LAYER(map, l)->class[0]->label.force = MS_TRUE;
@@ -652,7 +716,7 @@ int msEmbedLegend(mapObj *map, imageObj *img)
   if(map->legend.postlabelcache) /* add it directly to the image */
     msDrawMarkerSymbol(&map->symbolset, img, &point, GET_LAYER(map, l)->class[0]->styles[0], 1.0);
   else
-    msAddLabel(map, l, 0, NULL, &point, NULL, " ", 1.0, NULL);
+    msAddLabel(map, l, 0, NULL, &point, NULL, "", 1.0, NULL);
 
   /* Mark layer as deleted so that it doesn't interfere with html legends or with saving maps */
   GET_LAYER(map, l)->status = MS_DELETE;
