@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapowscommon.h 7804 2008-07-09 11:27:53Z tomkralidis $
+ * $Id: mapowscommon.h 10713 2010-11-09 14:14:57Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  OGC OWS Common Implementation include file
@@ -56,6 +56,9 @@
 
 #define MS_OWSCOMMON_OWS_110_NAMESPACE_URI      "http://www.opengis.net/ows/1.1"
 
+#define MS_OWSCOMMON_OWS_20_NAMESPACE_URI   "http://www.opengis.net/ows/2.0"
+#define MS_OWSCOMMON_OWS_20_SCHEMAS_LOCATION    "/ows/2.0/owsAll.xsd"
+
 /* OGC URNs */
 
 #define MS_OWSCOMMON_URN_OGC_CRS_4326 "urn:opengis:def:crs:OGC:2:84"
@@ -68,9 +71,28 @@
 
 #define MS_OWSCOMMON_OGC_CODESPACE "OGC"
 
+/* WCS namespaces */
+
+#define MS_OWSCOMMON_WCS_20_NAMESPACE_URI		"http://www.opengis.net/wcs/2.0"
+#define MS_OWSCOMMON_WCS_20_SCHEMAS_LOCATION    "/wcs/2.0/wcsAll.xsd"
+#define MS_OWSCOMMON_WCS_NAMESPACE_PREFIX		"wcs"
+
+/* GML namespaces */
+
+#define MS_OWSCOMMON_GML_32_NAMESPACE_URI		"http://www.opengis.net/gml/3.2"
+#define MS_OWSCOMMON_GML_NAMESPACE_PREFIX		"gml"
+
+/* GMLCov namespaces */
+
+#define MS_OWSCOMMON_GMLCOV_10_NAMESPACE_URI		"http://www.opengis.net/gmlcov/1.0"
+#define MS_OWSCOMMON_GMLCOV_NAMESPACE_PREFIX		"gmlcov"
+
+/* SWE namespaces */
+#define MS_OWSCOMMON_SWE_20_NAMESPACE_URI		"http://www.opengis.net/swe/2.0"
+#define MS_OWSCOMMON_SWE_NAMESPACE_PREFIX		"swe"
+
 /* function prototypes */
 
-int msOWSCommonNegotiateVersion(int requested_version, int supported_versions[], int num_supported_versions);
 
 xmlNodePtr msOWSCommonServiceIdentification(xmlNsPtr psNsOws, mapObj *map, const char *servicetype, const char *version, const char *namespaces);
 
@@ -94,6 +116,13 @@ xmlNodePtr msOWSCommonWGS84BoundingBox(xmlNsPtr psNsOws, int dimensions, double
 
 int _validateNamespace(xmlNsPtr psNsOws);
 
+xmlNodePtr msOWSCommonxmlNewChildEncoded( xmlNodePtr psParent, xmlNsPtr psNs, const char* name,
+                                          const char *content, const char *encoding);
+
+int msOWSSchemaValidation(const char* xml_schema, const char* xml);
+
 #endif /* defined(USE_LIBXML2) */
 
+int msOWSCommonNegotiateVersion(int requested_version, int supported_versions[], int num_supported_versions);
+
 #endif /* MAPOWSCOMMON_H */
