@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapproject.c 8022 2008-11-19 01:27:37Z warmerdam $
+ * $Id: mapproject.c 11376 2011-03-30 01:11:13Z dmorissette $
  *
  * Project:  MapServer
  * Purpose:  projectionObj / PROJ.4 interface.
@@ -31,8 +31,10 @@
 #include "mapproject.h"
 #include "mapthread.h"
 #include <assert.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
-MS_CVSID("$Id: mapproject.c 8022 2008-11-19 01:27:37Z warmerdam $")
+MS_CVSID("$Id: mapproject.c 11376 2011-03-30 01:11:13Z dmorissette $")
 
 #ifdef USE_PROJ
 static int msTestNeedWrap( pointObj pt1, pointObj pt2, pointObj pt2_geo,
@@ -194,138 +196,6 @@ static void msProjectGrowRect(projectionObj *in, projectionObj *out,
 }
 #endif /* def USE_PROJ */
 
-/************************************************************************/
-/*                           msProjectRect()                            */
-/************************************************************************/
-
-#define NUMBER_OF_SAMPLE_POINTS 100
-
-int msProjectRect(projectionObj *in, projectionObj *out, rectObj *rect) 
-{
-#ifdef USE_PROJ
-  pointObj prj_point;
-  rectObj prj_rect;
-  int	  rect_initialized = MS_FALSE, failure=0;
-  int     ix, iy;
-
-  double dx, dy;
-  double x, y;
-
-  dx = (rect->maxx - rect->minx)/NUMBER_OF_SAMPLE_POINTS;
-  dy = (rect->maxy - rect->miny)/NUMBER_OF_SAMPLE_POINTS;
-
-  /* first ensure the top left corner is processed, even if the rect
-     turns out to be degenerate. */
-
-  prj_point.x = rect->minx;
-  prj_point.y = rect->miny;
-#ifdef USE_POINT_Z_M
-  prj_point.z = 0.0;
-  prj_point.m = 0.0;
-#endif /* USE_POINT_Z_M */
-
-  msProjectGrowRect(in,out,&prj_rect,&rect_initialized,&prj_point,
-                    &failure);
-
-  /* sample along top and bottom */
-  if(dx > 0) {
-    for(ix = 0; ix <= NUMBER_OF_SAMPLE_POINTS; ix++ )
-    {
-      x = rect->minx + ix * dx;
-
-      prj_point.x = x;
-      prj_point.y = rect->miny;
-      msProjectGrowRect(in,out,&prj_rect,&rect_initialized,&prj_point,
-                        &failure);
-      
-      prj_point.x = x;
-      prj_point.y = rect->maxy;
-      msProjectGrowRect(in,out,&prj_rect,&rect_initialized,&prj_point,
-                        &failure);
-    }
-  }
-
-  /* sample along left and right */
-  if(dy > 0) {
-    for(iy = 0; iy <= NUMBER_OF_SAMPLE_POINTS; iy++ )
-    {
-      y = rect->miny + iy * dy;
-
-      prj_point.y = y;
-      prj_point.x = rect->minx;    
-      msProjectGrowRect(in,out,&prj_rect,&rect_initialized,&prj_point,
-                        &failure);
-      
-      prj_point.x = rect->maxx;
-      prj_point.y = y;
-      msProjectGrowRect(in,out,&prj_rect,&rect_initialized,&prj_point,
-                        &failure);
-    }
-  }
-
-  /*
-  ** If there have been any failures around the edges, then we had better
-  ** try and fill in the interior to get a close bounds. 
-  */
-  if( failure > 0 )
-  {
-      failure = 0;
-      for(ix = 0; ix <= NUMBER_OF_SAMPLE_POINTS; ix++ )
-      {
-          x = rect->minx + ix * dx;
-
-          for(iy = 0; iy <= NUMBER_OF_SAMPLE_POINTS; iy++ )
-          {
-              y = rect->miny + iy * dy;
-
-              prj_point.x = x;
-              prj_point.y = y;
-              msProjectGrowRect(in,out,&prj_rect,&rect_initialized,&prj_point,
-                                &failure);
-          }
-      }
-
-      if( !rect_initialized )
-      {
-          if( out == NULL || out->proj == NULL 
-              || pj_is_latlong(in->proj) )
-          {
-              prj_rect.minx = -180;
-              prj_rect.maxx = 180;
-              prj_rect.miny = -90;
-              prj_rect.maxy = 90;
-          }
-          else
-          {
-              prj_rect.minx = -22000000;
-              prj_rect.maxx = 22000000;
-              prj_rect.miny = -11000000;
-              prj_rect.maxy = 11000000;
-          }
-
-          msDebug( "msProjectRect(): all points failed to reproject, trying to fall back to using world bounds ... hope this helps.\n" );
-      }
-      else
-      {
-          msDebug( "msProjectRect(): some points failed to reproject, doing internal sampling.\n" );
-      }
-  }
-
-  rect->minx = prj_rect.minx;
-  rect->miny = prj_rect.miny;
-  rect->maxx = prj_rect.maxx;
-  rect->maxy = prj_rect.maxy;
-
-  if( !rect_initialized )
-      return MS_FAILURE;
-  else
-      return(MS_SUCCESS);
-#else
-  msSetError(MS_PROJERR, "Projection support is not available.", "msProjectRect()");
-  return(MS_FAILURE);
-#endif
-}
-
 /************************************************************************/
 /*                          msProjectSegment()                          */
 /*                                                                      */
@@ -418,7 +288,7 @@ msProjectShapeLine(projectionObj *in, projectionObj *out,
     int line_alloc = numpoints_in;
     int wrap_test;
 
-    wrap_test = out->proj != NULL && pj_is_latlong(out->proj)
+    wrap_test = out != NULL && out->proj != NULL && pj_is_latlong(out->proj)
         && !pj_is_latlong(in->proj);
 
     line->numpoints = 0;
@@ -583,7 +453,9 @@ msProjectShapeLine(projectionObj *in, projectionObj *out,
         && (line_out->point[0].x != line_out->point[line_out->numpoints-1].x
             || line_out->point[0].y != line_out->point[line_out->numpoints-1].y) )
     {
-        msAddPointToLine( line_out, line_out->point + 0 );
+        /* make a copy because msAddPointToLine can realloc the array */
+        pointObj sFirstPoint = line_out->point[0];
+        msAddPointToLine( line_out, &sFirstPoint );
     }
 
     return(MS_SUCCESS);
@@ -696,6 +568,318 @@ int msProjectLine(projectionObj *in, projectionObj *out, lineObj *line)
 #endif
 }
 
+/************************************************************************/
+/*                           msProjectRectGrid()                        */
+/************************************************************************/
+
+#define NUMBER_OF_SAMPLE_POINTS 100
+
+int msProjectRectGrid(projectionObj *in, projectionObj *out, rectObj *rect) 
+{
+#ifdef USE_PROJ
+  pointObj prj_point;
+  rectObj prj_rect;
+  int	  rect_initialized = MS_FALSE, failure=0;
+  int     ix, iy;
+
+  double dx, dy;
+  double x, y;
+
+  dx = (rect->maxx - rect->minx)/NUMBER_OF_SAMPLE_POINTS;
+  dy = (rect->maxy - rect->miny)/NUMBER_OF_SAMPLE_POINTS;
+
+  /* first ensure the top left corner is processed, even if the rect
+     turns out to be degenerate. */
+
+  prj_point.x = rect->minx;
+  prj_point.y = rect->miny;
+#ifdef USE_POINT_Z_M
+  prj_point.z = 0.0;
+  prj_point.m = 0.0;
+#endif /* USE_POINT_Z_M */
+
+  msProjectGrowRect(in,out,&prj_rect,&rect_initialized,&prj_point,
+                    &failure);
+
+  failure = 0;
+  for(ix = 0; ix <= NUMBER_OF_SAMPLE_POINTS; ix++ )
+  {
+      x = rect->minx + ix * dx;
+      
+      for(iy = 0; iy <= NUMBER_OF_SAMPLE_POINTS; iy++ )
+      {
+          y = rect->miny + iy * dy;
+          
+          prj_point.x = x;
+          prj_point.y = y;
+          msProjectGrowRect(in,out,&prj_rect,&rect_initialized,&prj_point,
+                            &failure);
+      }
+  }
+  
+  if( !rect_initialized )
+  {
+      if( out == NULL || out->proj == NULL 
+          || pj_is_latlong(in->proj) )
+      {
+          prj_rect.minx = -180;
+          prj_rect.maxx = 180;
+          prj_rect.miny = -90;
+          prj_rect.maxy = 90;
+      }
+      else
+      {
+          prj_rect.minx = -22000000;
+          prj_rect.maxx = 22000000;
+          prj_rect.miny = -11000000;
+          prj_rect.maxy = 11000000;
+      }
+      
+      msDebug( "msProjectRect(): all points failed to reproject, trying to fall back to using world bounds ... hope this helps.\n" );
+  }
+  else
+  {
+      msDebug( "msProjectRect(): some points failed to reproject, doing internal sampling.\n" );
+  }
+
+  rect->minx = prj_rect.minx;
+  rect->miny = prj_rect.miny;
+  rect->maxx = prj_rect.maxx;
+  rect->maxy = prj_rect.maxy;
+
+  if( !rect_initialized )
+      return MS_FAILURE;
+  else
+      return(MS_SUCCESS);
+#else
+  msSetError(MS_PROJERR, "Projection support is not available.", "msProjectRect()");
+  return(MS_FAILURE);
+#endif
+}
+
+/************************************************************************/
+/*                    msProjectRectTraditionalEdge()                    */
+/************************************************************************/
+#ifdef notdef
+static int 
+msProjectRectTraditionalEdge(projectionObj *in, projectionObj *out, 
+                             rectObj *rect) 
+{
+#ifdef USE_PROJ
+  pointObj prj_point;
+  rectObj prj_rect;
+  int	  rect_initialized = MS_FALSE, failure=0;
+  int     ix, iy;
+
+  double dx, dy;
+  double x, y;
+
+  dx = (rect->maxx - rect->minx)/NUMBER_OF_SAMPLE_POINTS;
+  dy = (rect->maxy - rect->miny)/NUMBER_OF_SAMPLE_POINTS;
+
+  /* first ensure the top left corner is processed, even if the rect
+     turns out to be degenerate. */
+
+  prj_point.x = rect->minx;
+  prj_point.y = rect->miny;
+#ifdef USE_POINT_Z_M
+  prj_point.z = 0.0;
+  prj_point.m = 0.0;
+#endif /* USE_POINT_Z_M */
+
+  msProjectGrowRect(in,out,&prj_rect,&rect_initialized,&prj_point,
+                    &failure);
+
+  /* sample along top and bottom */
+  if(dx > 0) {
+    for(ix = 0; ix <= NUMBER_OF_SAMPLE_POINTS; ix++ )
+    {
+      x = rect->minx + ix * dx;
+
+      prj_point.x = x;
+      prj_point.y = rect->miny;
+      msProjectGrowRect(in,out,&prj_rect,&rect_initialized,&prj_point,
+                        &failure);
+      
+      prj_point.x = x;
+      prj_point.y = rect->maxy;
+      msProjectGrowRect(in,out,&prj_rect,&rect_initialized,&prj_point,
+                        &failure);
+    }
+  }
+
+  /* sample along left and right */
+  if(dy > 0) {
+    for(iy = 0; iy <= NUMBER_OF_SAMPLE_POINTS; iy++ )
+    {
+      y = rect->miny + iy * dy;
+
+      prj_point.y = y;
+      prj_point.x = rect->minx;    
+      msProjectGrowRect(in,out,&prj_rect,&rect_initialized,&prj_point,
+                        &failure);
+      
+      prj_point.x = rect->maxx;
+      prj_point.y = y;
+      msProjectGrowRect(in,out,&prj_rect,&rect_initialized,&prj_point,
+                        &failure);
+    }
+  }
+
+  /*
+  ** If there have been any failures around the edges, then we had better
+  ** try and fill in the interior to get a close bounds. 
+  */
+  if( failure > 0 )
+      return msProjectRectGrid( in, out, rect );
+
+  rect->minx = prj_rect.minx;
+  rect->miny = prj_rect.miny;
+  rect->maxx = prj_rect.maxx;
+  rect->maxy = prj_rect.maxy;
+
+  if( !rect_initialized )
+      return MS_FAILURE;
+  else
+      return(MS_SUCCESS);
+#else
+  msSetError(MS_PROJERR, "Projection support is not available.", "msProjectRect()");
+  return(MS_FAILURE);
+#endif
+}
+#endif /* def notdef */
+
+/************************************************************************/
+/*                       msProjectRectAsPolygon()                       */
+/************************************************************************/
+
+static int 
+msProjectRectAsPolygon(projectionObj *in, projectionObj *out, 
+                       rectObj *rect) 
+{
+#ifdef USE_PROJ
+  shapeObj polygonObj;
+  lineObj  ring;
+//  pointObj ringPoints[NUMBER_OF_SAMPLE_POINTS*4+4];
+  pointObj *ringPoints;
+  int     ix, iy;
+
+  double dx, dy;
+
+  ringPoints = (pointObj*) calloc(sizeof(pointObj),NUMBER_OF_SAMPLE_POINTS*4+4);
+  ring.point = ringPoints;
+  ring.numpoints = 0;
+
+  msInitShape( &polygonObj );
+  polygonObj.type = MS_SHAPE_POLYGON;
+
+/* -------------------------------------------------------------------- */
+/*      Build polygon as steps around the source rectangle.             */
+/* -------------------------------------------------------------------- */
+  dx = (rect->maxx - rect->minx)/NUMBER_OF_SAMPLE_POINTS;
+  dy = (rect->maxy - rect->miny)/NUMBER_OF_SAMPLE_POINTS;
+
+  /* sample along top */
+  if(dx != 0) {
+    for(ix = 0; ix <= NUMBER_OF_SAMPLE_POINTS; ix++ )
+    {
+      ringPoints[ring.numpoints].x = rect->minx + ix * dx;
+      ringPoints[ring.numpoints++].y = rect->miny;
+    }
+  }
+
+  /* sample on along right side */
+  if(dy != 0) {
+    for(iy = 1; iy <= NUMBER_OF_SAMPLE_POINTS; iy++ )
+    {
+      ringPoints[ring.numpoints].x = rect->maxx;
+      ringPoints[ring.numpoints++].y = rect->miny + iy * dy;
+    }
+  }
+
+  /* sample along bottom */
+  if(dx != 0) {
+    for(ix = NUMBER_OF_SAMPLE_POINTS-1; ix >= 0; ix-- )
+    {
+      ringPoints[ring.numpoints].x = rect->minx + ix * dx;
+      ringPoints[ring.numpoints++].y = rect->maxy;
+    }
+  }
+
+  /* sample on along left side */
+  if(dy != 0) {
+    for(iy = NUMBER_OF_SAMPLE_POINTS-1; iy >= 0; iy-- )
+    {
+      ringPoints[ring.numpoints].x = rect->minx;
+      ringPoints[ring.numpoints++].y = rect->miny + iy * dy;
+    }
+  }
+
+  msAddLineDirectly( &polygonObj, &ring );
+
+/* -------------------------------------------------------------------- */
+/*      Attempt to reproject.                                           */
+/* -------------------------------------------------------------------- */
+  msProjectShapeLine( in, out, &polygonObj, 0 );
+
+  /* If no points reprojected, try a grid sampling */
+  if( polygonObj.numlines == 0 || polygonObj.line[0].numpoints == 0 )
+  {
+      msFreeShape( &polygonObj );
+      return msProjectRectGrid( in, out, rect );
+  }
+
+/* -------------------------------------------------------------------- */
+/*      Collect bounds.                                                 */
+/* -------------------------------------------------------------------- */
+  rect->minx = rect->maxx = polygonObj.line[0].point[0].x;
+  rect->miny = rect->maxy = polygonObj.line[0].point[0].y;
+  
+  for( ix = 1; ix < polygonObj.line[0].numpoints; ix++ )
+  {
+      pointObj  *pnt = polygonObj.line[0].point + ix;
+
+      rect->minx = MS_MIN(rect->minx,pnt->x);
+      rect->maxx = MS_MAX(rect->maxx,pnt->x);
+      rect->miny = MS_MIN(rect->miny,pnt->y);
+      rect->maxy = MS_MAX(rect->maxy,pnt->y);
+  }
+
+  msFreeShape( &polygonObj );
+
+/* -------------------------------------------------------------------- */
+/*      Special case to handle reprojection from "more than the         */
+/*      whole world" projected coordinates that sometimes produce a     */
+/*      region greater than 360 degrees wide due to various wrapping    */
+/*      logic.                                                          */
+/* -------------------------------------------------------------------- */
+  if( out && pj_is_latlong(out->proj) && in && !pj_is_latlong(in->proj) 
+      && rect->maxx - rect->minx > 360.0 )
+  {
+      rect->maxx = 180;
+      rect->minx = -180;
+  }
+
+  return MS_SUCCESS;
+#else
+  msSetError(MS_PROJERR, "Projection support is not available.", "msProjectRect()");
+  return(MS_FAILURE);
+#endif
+}
+
+/************************************************************************/
+/*                           msProjectRect()                            */
+/************************************************************************/
+
+int msProjectRect(projectionObj *in, projectionObj *out, rectObj *rect) 
+{
+#ifdef notdef
+    return msProjectRectTraditionalEdge( in, out, rect );
+#else
+    return msProjectRectAsPolygon( in, out, rect );
+#endif    
+}
+
 /************************************************************************/
 /*                        msProjectionsDiffer()                         */
 /************************************************************************/
@@ -871,11 +1055,33 @@ static const char *msProjFinder( const char *filename)
 /************************************************************************/
 /*                           msSetPROJ_LIB()                            */
 /************************************************************************/
-void msSetPROJ_LIB( const char *proj_lib )
+void msSetPROJ_LIB( const char *proj_lib, const char *pszRelToPath )
 
 {
 #ifdef USE_PROJ
     static int finder_installed = 0;
+    char *extended_path = NULL;
+
+    /* Handle relative path if applicable */
+    if( proj_lib && pszRelToPath
+        && proj_lib[0] != '/'
+        && proj_lib[0] != '\\'
+        && !(proj_lib[0] != '\0' && proj_lib[1] == ':') )
+    {
+        struct stat stat_buf;
+        extended_path = (char*) msSmallMalloc(strlen(pszRelToPath)
+                                              + strlen(proj_lib) + 10);
+        sprintf( extended_path, "%s/%s", pszRelToPath, proj_lib );
+
+#ifndef S_ISDIR
+#  define S_ISDIR(x) ((x) & S_IFDIR)
+#endif            
+            
+        if( stat( extended_path, &stat_buf ) == 0 
+            && S_ISDIR(stat_buf.st_mode) )
+            proj_lib = extended_path;
+    }
+
 
     msAcquireLock( TLOCK_PROJ );
 
@@ -900,9 +1106,12 @@ void msSetPROJ_LIB( const char *proj_lib )
     }
 
     if( proj_lib != NULL )
-        ms_proj_lib = strdup( proj_lib );
+        ms_proj_lib = msStrdup( proj_lib );
     
     msReleaseLock( TLOCK_PROJ );
+
+    if ( extended_path )
+        msFree( extended_path );
 #endif
 }
 
