@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapprojhack.c 12509 2011-09-01 14:03:03Z hobu $
+ * $Id: mapprojhack.c 11832 2011-06-17 19:06:58Z warmerdam $
  *
  * Project:  MapServer
  * Purpose:  Implementation of utility functions related to the PROJ4 library
@@ -27,7 +27,11 @@
  ****************************************************************************/
 
 #include <string.h>
-#include "mapserver.h"
+#include "mapproject.h"
+
+enum MS_UNITS {MS_INCHES, MS_FEET, MS_MILES, MS_METERS, MS_KILOMETERS,
+               MS_DD, MS_PIXELS, MS_NAUTICALMILES};
+
 
 /************************************************************************/
 /*                        ConvertProjUnitStringToMS                     */
@@ -92,7 +96,7 @@ int GetMapserverUnitUsingProj(projectionObj *psProj)
         char units[32];
         char *blank;
 
-        strlcpy( units, (strstr(proj_str,"units=")+6), sizeof(units) );
+        strncpy( units, (strstr(proj_str,"units=")+6), sizeof(units)-2 );
         pj_dalloc( proj_str );
 
         blank = strchr(units, ' ');
@@ -111,8 +115,8 @@ int GetMapserverUnitUsingProj(projectionObj *psProj)
         char *blank;
         double to_meter;
 
-        strlcpy(to_meter_str,(strstr(proj_str,"to_meter=")+9),
-                sizeof(to_meter_str));
+        strncpy(to_meter_str,(strstr(proj_str,"to_meter=")+9),
+                sizeof(to_meter_str)-2);
         pj_dalloc( proj_str );
 
         blank = strchr(to_meter_str, ' ');
