@@ -1,4 +1,4 @@
-# $Id: shapetest.py 5121 2006-01-18 04:34:35Z frank $
+# $Id: shapetest.py 6523 2007-08-08 20:50:12Z hobu $
 #
 # Project:  MapServer
 # Purpose:  xUnit style Python mapscript tests of Shape
@@ -98,11 +98,12 @@ class ShapeValuesTestCase(unittest.TestCase):
 
     def testNullValue(self):
         so = mapscript.shapeObj(mapscript.MS_SHAPE_POINT)
-        assert so.numvalues == 4
-        assert so.getValue(0) == ''
+        assert so.numvalues == 0
+        assert so.getValue(0) == None
     
     def testSetValue(self):
         so = mapscript.shapeObj(mapscript.MS_SHAPE_POINT)
+        so.initValues(4)
         so.setValue(0, 'Foo');
         assert so.numvalues == 4
         assert so.getValue(0) == 'Foo'
@@ -112,8 +113,8 @@ class ShapeValuesTestCase(unittest.TestCase):
 class ShapeWKTTestCase(unittest.TestCase):
     
     # define a pair of coords, and WKT as class data
-    point_xy = (-105.5, 40.0)
-    point_wkt = 'POINT (-105.5 40.0)'
+    point_xy = (-105.5000000000000000, 40.0000000000000000)
+    point_wkt = 'POINT (-105.5000000000000000 40.0000000000000000)'
     
     def testSetPointWKT(self):
         # Create new instance and set/init from WKT
