@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: shptree.c 8015 2008-11-11 18:14:14Z pramsey $
+ * $Id: shptree.c 10772 2010-11-29 18:27:02Z aboudreault $
  *
  * Project:  MapServer
  * Purpose:  Commandline utility to generate .qix shapefile spatial indexes.
@@ -31,7 +31,7 @@
 #include "maptree.h"
 #include <string.h>
 
-MS_CVSID("$Id: shptree.c 8015 2008-11-11 18:14:14Z pramsey $")
+MS_CVSID("$Id: shptree.c 10772 2010-11-29 18:27:02Z aboudreault $")
 
 char* AddFileSuffix ( const char * Filename, const char * Suffix ) {
   char	*pszFullname, *pszBasename;
@@ -41,7 +41,7 @@ char* AddFileSuffix ( const char * Filename, const char * Suffix ) {
   /*	Compute the base (layer) name.  If there is any extension	    */
   /*	on the passed in filename we will strip it off.			    */
   /* -------------------------------------------------------------------- */
-  pszBasename = (char *) malloc(strlen(Filename)+5);
+  pszBasename = (char *) msSmallMalloc(strlen(Filename)+5);
   strcpy( pszBasename, Filename );
   for( i = strlen(pszBasename)-1; 
        i > 0 && pszBasename[i] != '.' && pszBasename[i] != '/'
@@ -55,7 +55,7 @@ char* AddFileSuffix ( const char * Filename, const char * Suffix ) {
   /*	Open the .shp and .shx files.  Note that files pulled from	    */
   /*	a PC to Unix with upper case filenames won't work!		    */
   /* -------------------------------------------------------------------- */
-  pszFullname = (char *) malloc(strlen(pszBasename) + 5);
+  pszFullname = (char *) msSmallMalloc(strlen(pszBasename) + 5);
   sprintf( pszFullname, "%s%s", pszBasename, Suffix); 
   
   free(pszBasename);
